"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LinesAndColumns = exports.Context = void 0;
const lodash_1 = __importDefault(require("lodash"));
const template_scope_manager_1 = require("./template-scope-manager");
class Context {
    constructor(code, parserOptions) {
        this.tokens = [];
        this.comments = [];
        this.locsMap = new Map();
        this._templateScopeManager = null;
        this.code = code;
        this.parserOptions = parserOptions;
        this.locs = new LinesAndColumns(code);
        let templateCode = "";
        let scriptCode = "";
        let scriptAttrs = {};
        let start = 0;
        for (const block of extractBlocks(code)) {
            const before = code.slice(start, block.codeRange[0]);
            const blankCode = block.code.replace(/[^\n\r ]/g, " ");
            templateCode += before + blankCode;
            if (block.tag === "script") {
                scriptCode += before.replace(/[^\n\r ]/g, " ") + block.code;
                scriptAttrs = Object.assign(scriptAttrs, block.attrs);
            }
            else {
                scriptCode += before.replace(/[^\n\r ]/g, " ") + blankCode;
            }
            start = block.codeRange[1];
        }
        const before = code.slice(start);
        templateCode += before;
        scriptCode += before.replace(/[^\n\r ]/g, " ");
        this.sourceCode = {
            template: templateCode,
            scripts: {
                code: scriptCode,
                attrs: scriptAttrs,
            },
        };
    }
    getLocFromIndex(index) {
        let loc = this.locsMap.get(index);
        if (!loc) {
            loc = this.locs.getLocFromIndex(index);
            this.locsMap.set(index, loc);
        }
        return {
            line: loc.line,
            column: loc.column,
        };
    }
    /**
     * Get the location information of the given node.
     * @param node The node.
     */
    getConvertLocation(node) {
        const { start, end } = node;
        return {
            range: [start, end],
            loc: {
                start: this.getLocFromIndex(start),
                end: this.getLocFromIndex(end),
            },
        };
    }
    addComment(comment) {
        this.comments.push(comment);
    }
    /**
     * Add token to tokens
     */
    addToken(type, range) {
        const token = Object.assign({ type, value: this.getText(range) }, this.getConvertLocation(range));
        this.tokens.push(token);
        return token;
    }
    /**
     * get text
     */
    getText(range) {
        return this.code.slice(range.start, range.end);
    }
    readyScopeManager(scopeManager) {
        this._templateScopeManager = new template_scope_manager_1.TemplateScopeManager(scopeManager);
    }
    get templateScopeManager() {
        return this._templateScopeManager;
    }
}
exports.Context = Context;
/** Extract <script> blocks */
function* extractBlocks(code) {
    const startTagRe = /<(script|style)(\s[\s\S]*?)?>/giu;
    const endScriptTagRe = /<\/script(?:\s[\s\S]*?)?>/giu;
    const endStyleTagRe = /<\/style(?:\s[\s\S]*?)?>/giu;
    let startTagRes;
    while ((startTagRes = startTagRe.exec(code))) {
        const [startTag, tag, attributes = ""] = startTagRes;
        const startTagStart = startTagRes.index;
        const startTagEnd = startTagStart + startTag.length;
        const endTagRe = tag.toLowerCase() === "script" ? endScriptTagRe : endStyleTagRe;
        endTagRe.lastIndex = startTagRe.lastIndex;
        const endTagRes = endTagRe.exec(code);
        if (endTagRes) {
            const endTagStart = endTagRes.index;
            const codeRange = [startTagEnd, endTagStart];
            const attrRe = 
            // eslint-disable-next-line regexp/no-unused-capturing-group -- maybe bug
            /(<key>[^\s=]+)(?:=(?:"(<val>[^"]*)"|'(<val>[^"]*)'|(<val>[^\s=]+)))?/giu;
            const attrs = {};
            let attrRes;
            while ((attrRes = attrRe.exec(attributes))) {
                attrs[attrRes.groups.key] = attrRes.groups.val;
            }
            yield {
                code: code.slice(...codeRange),
                codeRange,
                attrs,
                tag: tag,
            };
            startTagRe.lastIndex = endTagRe.lastIndex;
        }
    }
}
class LinesAndColumns {
    constructor(code) {
        const len = code.length;
        const lineStartIndices = [0];
        for (let index = 0; index < len; index++) {
            const c = code[index];
            if (c === "\r") {
                const next = code[index + 1] || "";
                if (next === "\n") {
                    index++;
                }
                lineStartIndices.push(index + 1);
            }
            else if (c === "\n") {
                lineStartIndices.push(index + 1);
            }
        }
        this.lineStartIndices = lineStartIndices;
    }
    getLocFromIndex(index) {
        const lineNumber = lodash_1.default.sortedLastIndex(this.lineStartIndices, index);
        return {
            line: lineNumber,
            column: index - this.lineStartIndices[lineNumber - 1],
        };
    }
    getIndexFromLoc(loc) {
        const lineStartIndex = this.lineStartIndices[loc.line - 1];
        const positionIndex = lineStartIndex + loc.column;
        return positionIndex;
    }
}
exports.LinesAndColumns = LinesAndColumns;
